export interface TypeSocketOptions {
    maxRetries?: number;
    retryOnClose?: boolean;
    retryTime?: number;
}
export declare class TypeSocket<T> {
    private url;
    onConnected?: () => void;
    onDisconnected?: () => void;
    onPermanentlyDisconnected?: () => void;
    onMessage?: (message: T) => void;
    private socket;
    private retries;
    private options;
    /**
     * Creates a new TypeSocket
     * @param url WebSocket server URL
     */
    constructor(url: string, options?: TypeSocketOptions);
    /**
     * Connects to the server.
     */
    connect(): void;
    /**
     * Sends a JavaScript object of type T to the server.
     * @param data JS object.
     */
    send(data: T): void;
    /**
     * Ready state of the socket.
     */
    get readyState(): number;
    private reconnect;
    private connected;
    private disconnected;
    private permanentlyDisconnected;
    private message;
}
