"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
Object.defineProperty(exports, "__esModule", { value: true });
;
var TypeSocket = /** @class */ (function () {
    /**
     * Creates a new TypeSocket
     * @param url WebSocket server URL
     */
    function TypeSocket(url, options) {
        this.url = url;
        this.socket = null;
        this.retries = 0;
        this.options = {
            maxRetries: 5,
            retryOnClose: false,
            retryTime: 500,
        };
        this.connect();
        if (options) {
            this.options = __assign(__assign({}, this.options), options);
        }
    }
    /**
     * Connects to the server.
     */
    TypeSocket.prototype.connect = function () {
        var _this = this;
        if (this.socket) {
            try {
                this.socket.close();
                this.disconnected();
            }
            catch (_a) { }
        }
        this.socket = new WebSocket(this.url);
        this.socket.onopen = function () {
            _this.connected();
        };
        this.socket.onclose = function (e) {
            _this.disconnected();
            _this.socket = null;
            if (e.code === 1000 || _this.options.retryOnClose) {
                setTimeout(function () {
                    _this.reconnect();
                }, _this.options.retryTime);
            }
            else {
                _this.permanentlyDisconnected();
            }
        };
        this.socket.onmessage = function (e) {
            _this.message(e.data);
        };
        this.socket.onerror = function () {
            _this.disconnected();
            _this.socket = null;
            setTimeout(function () {
                _this.reconnect();
            }, _this.options.retryTime);
        };
    };
    /**
     * Sends a JavaScript object of type T to the server.
     * @param data JS object.
     */
    TypeSocket.prototype.send = function (data) {
        if (!this.socket)
            return;
        this.socket.send(JSON.stringify(data));
    };
    Object.defineProperty(TypeSocket.prototype, "readyState", {
        /**
         * Ready state of the socket.
         */
        get: function () {
            return this.socket ? this.socket.readyState : 0;
        },
        enumerable: true,
        configurable: true
    });
    TypeSocket.prototype.reconnect = function () {
        this.retries++;
        if (this.options.maxRetries && this.retries > this.options.maxRetries) {
            this.disconnected();
            this.permanentlyDisconnected();
            return;
        }
        this.connect();
    };
    TypeSocket.prototype.connected = function () {
        this.retries = 0;
        if (this.onConnected) {
            this.onConnected();
        }
    };
    TypeSocket.prototype.disconnected = function () {
        if (this.onDisconnected) {
            this.onDisconnected();
        }
    };
    TypeSocket.prototype.permanentlyDisconnected = function () {
        if (this.onPermanentlyDisconnected) {
            this.onPermanentlyDisconnected();
        }
    };
    TypeSocket.prototype.message = function (data) {
        try {
            var json = JSON.parse(data);
            if (json && this.onMessage) {
                this.onMessage(json);
            }
        }
        catch (_a) { }
    };
    return TypeSocket;
}());
exports.TypeSocket = TypeSocket;
;
